package b4a.SM.PopupWindow;

import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.objects.ActivityWrapper;

import android.graphics.drawable.Drawable;
import android.os.Build;
import android.transition.Transition;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.widget.PopupWindow;

import android.content.res.ColorStateList;
import android.graphics.drawable.ColorDrawable;
import android.graphics.drawable.RippleDrawable;
import android.graphics.drawable.StateListDrawable;

/**
    * This class represents a popup window that can be used to display an arbitrary view. The popup window is a floating container that appears on top of the current activity.
    */

@BA.Version (1.01f)
@BA.ShortName ("smPopupWindow")
@BA.ActivityObject

public class smPopupWindow
    {
    public static final int INPUT_METHOD_FROM_FOCUSABLE           = 0x00000000;
    public static final int INPUT_METHOD_NEEDED                   = 0x00000001;
    public static final int INPUT_METHOD_NOT_NEEDED               = 0x00000002;

    private PopupWindow                               mPopupWindow = null;
    private String                                    mEventName  = "";
    private boolean                                   mIsMenu     = false;
    private anywheresoftware.b4a.objects.PanelWrapper mPanel      = null;

/**
    * Creates a new popup window which displays the B4A Panel.
    * Initial settings : focusable; touchable; outsidetouchable; elevation (API 21+) = 4dip.
    * isMenu = true means dismiss the popup window when touching outside the window.
    */
    public void Initialize (final BA ba, String eventName, anywheresoftware.b4a.objects.PanelWrapper panel, boolean isMenu)
        {
        mEventName = eventName.toLowerCase (BA.cul);
        mPanel = panel;
        mIsMenu = isMenu;
        android.view.View contentView = (android.view.View) panel.getObject ();
        int width  = panel.getWidth  ();
        int height = panel.getHeight ();
        mPopupWindow = new PopupWindow (contentView, width, height);
        mPopupWindow.setFocusable (true);
        mPopupWindow.setTouchable (true);
        mPopupWindow.setOutsideTouchable (true);
        setElevation (ba, (float) (anywheresoftware.b4a.keywords.Common.DipToCurrent ((int) (4))));
        mPopupWindow.setOnDismissListener
            (
            new PopupWindow.OnDismissListener ()
                {
                @Override
                public void onDismiss ()
                    {
                    if (ba.subExists (mEventName + "_ondismiss"))
                        {
                        ba.raiseEvent (ba, mEventName + "_ondismiss");
                        }
                    }
                }
            );
        contentView.setOnTouchListener
            (
            new View.OnTouchListener ()
                {
                @Override
                public boolean onTouch (View v, MotionEvent event)
                    {
                    if ((event.getX () < 0) || (event.getX () >= mPanel.getWidth ()) || (event.getY () < 0) || (event.getY () > mPanel.getHeight ()))
                        {
                        if (mIsMenu)
                           {
                           mPopupWindow.dismiss ();
                           return true;
                           }
                        }
                    return false;
                    }
                }
            );
        }

/**
    * Disposes of the popup window. This method can be invoked only after showAsDropDown (android.view.View) has been executed. Failing that, calling this method will have no effect.
    */
    public void dismiss (BA ba)
        {
        mPopupWindow.dismiss ();
        }

/**
    * Returns the animation style to use the popup appears and disappears.
    */
    public int getAnimationStyle (BA ba)
        {
        return mPopupWindow.getAnimationStyle ();
        }

/**
    * Returns the drawable used as the popup window's background.
    */
    public Drawable getBackground (BA ba)
        {
        return mPopupWindow.getBackground ();
        }

/**
    * Returns the view used as the content of the popup window.
    */
    public android.view.View getContentView (BA ba)
        {
        return mPopupWindow.getContentView ();
        }

/**
    * (API 21+) Returns the the elevation for this popup window in pixels.
    */
    public float getElevation (BA ba)
        {
        if (Build.VERSION.SDK_INT >= 21)
            {
            return mPopupWindow.getElevation ();
            }
        else
            {
            return 0.0f;
            }
        }

/**
    * (API 24+) Returns the enter transition to be used when the popup window is shown.
    */
    public Transition getEnterTransition (BA ba)
        {
        if (Build.VERSION.SDK_INT >= 24)
            {
            return mPopupWindow.getEnterTransition ();
            }
        else
            {
            return null;
            }
        }

/**
    * (API 24+) Returns the exit transition to be used when the popup window is dismissed.
    */
    public Transition getExitTransition (BA ba)
        {
        if (Build.VERSION.SDK_INT >= 24)
            {
            return mPopupWindow.getExitTransition ();
            }
        else
            {
            return null;
            }
        }

/**
    * Returns the popup's requested height. May be a layout constant such as ViewGroup.LayoutParams.WRAP_CONTENT or ViewGroup.LayoutParams.MATCH_PARENT. The actual size of the popup may depend on other factors such as clipping and window layout.
    */
    public int getHeight (BA ba)
        {
        return mPopupWindow.getHeight ();
        }

/**
    * Returns the current value in setInputMethodMode.
    */
    public int getInputMethodMode (BA ba)
        {
        return mPopupWindow.getInputMethodMode ();
        }

/**
    * Returns the maximum height that is available for the popup to be completely shown, optionally ignoring any bottom decorations such as the input method. It is recommended that this height be the maximum for the popup's height, otherwise it is possible that the popup will be clipped.
    * yOffset: y offset from the view's bottom edge.
    * ignoreBottomDecorations (API 24+) : if true, the height returned will be all the way to the bottom of the display, ignoring any bottom decorations.
    * ignoreBottomDecorations (API 23-) : not used.
    */
    public int getMaxAvailableHeight (BA ba, android.view.View anchor, int yOffset, boolean ignoreBottomDecorations)
        {
        if (Build.VERSION.SDK_INT >= 24)
            {
            return mPopupWindow.getMaxAvailableHeight (anchor, yOffset, ignoreBottomDecorations);
            }
        else
            {
            return mPopupWindow.getMaxAvailableHeight (anchor, yOffset);
            }
        }

/**
    * (API 23+) Returns whether the popup window should overlap its anchor view when displayed as a drop-down.
    */
    public boolean getOverlapAnchor (BA ba)
        {
        if (Build.VERSION.SDK_INT >= 23)
            {
            return mPopupWindow.getOverlapAnchor ();
            }
        else
            {
            return false;
            }
        }

/**
    * Returns the current value in setSoftInputMode.
    */
    public int getSoftInputMode (BA ba)
        {
        return mPopupWindow.getSoftInputMode ();
        }

/**
    * Returns the popup's requested width in pixels or a layout constant such as ViewGroup.LayoutParams.WRAP_CONTENT or ViewGroup.LayoutParams.MATCH_PARENT. The actual size of the popup may depend on other factors such as clipping and window layout.
    */
    public int getWidth (BA ba)
        {
        return mPopupWindow.getWidth ();
        }

/**
    * (API 23+) Returns the layout type for this window.
    */
    public int getWindowLayoutType (BA ba)
        {
        if (Build.VERSION.SDK_INT >= 23)
            {
            return mPopupWindow.getWindowLayoutType ();
            }
        else
            {
            return 0;
            }
        }

/**
    * Indicates whether the popup is showing above (the y coordinate of the popup's bottom is less than the y coordinate of the anchor) or below the anchor view (the y coordinate of the popup is greater than y coordinate of the anchor's bottom). The value returned by this method is meaningful only after showAsDropDown (android.view.View) or showAsDropDown (android.view.View, int, int) was invoked.
    */
    public boolean isAboveAnchor (BA ba)
        {
        return mPopupWindow.isAboveAnchor ();
        }

/**
    * (API 22+) Indicates whether the popup window will be attached in the decor frame of its parent window.
    */
    public boolean isAttachedInDecor (BA ba)
        {
        if (Build.VERSION.SDK_INT >= 22)
            {
            return mPopupWindow.isAttachedInDecor ();
            }
        else
            {
            return false;
            }
        }

/**
    * Indicates whether clipping of the popup window is enabled.
    */
    public boolean isClippingEnabled (BA ba)
        {
        return mPopupWindow.isClippingEnabled ();
        }

/**
    * Indicate whether the popup window can grab the focus.
    */
    public boolean isFocusable (BA ba)
        {
        return mPopupWindow.isFocusable ();
        }

/**
    * Indicates whether the popup window will be informed of touch events outside of its window.
    */
    public boolean isOutsideTouchable (BA ba)
        {
        return mPopupWindow.isOutsideTouchable ();
        }

/**
    * Indicate whether this popup window is showing on screen.
    */
    public boolean isShowing (BA ba)
        {
        return mPopupWindow.isShowing ();
        }

/**
    * Indicates whether the popup window supports splitting touches.
    */
    public boolean isSplitTouchEnabled (BA ba)
        {
        return mPopupWindow.isSplitTouchEnabled ();
        }

/**
    * Indicates whether the popup window receives touch events.
    */
    public boolean isTouchable (BA ba)
        {
        return mPopupWindow.isTouchable ();
        }


/**
    * Changes the animation style resource for this popup. Set to -1 for the default animation, 0 for no animation, or a resource identifier for an explicit animation.
    * If the popup is showing, calling this method will take effect only the next time the popup is shown or through a manual call to one of the update() methods.
    */
    public void setAnimationStyle (BA ba, int animationStyle)
        {
        mPopupWindow.setAnimationStyle (animationStyle);
        }

/**
    * (API 22+) This will attach the popup window to the decor frame of the parent window to avoid overlaping with screen decorations like the navigation bar.
    * Overrides the default behavior of the flag WindowManager.LayoutParams.FLAG_LAYOUT_ATTACHED_IN_DECOR. By default the flag is set on SDK version Build.VERSION_CODES.LOLLIPOP_MR1 or greater and cleared on lesser SDK versions.
    */
    public void setAttachedInDecor (BA ba, boolean enabled)
        {
        if (Build.VERSION.SDK_INT >= 22)
            {
            mPopupWindow.setAttachedInDecor (enabled);
            }
        }

/**
    * Specifies the background drawable for this popup window. The background can be set to null.
    */
    public void setBackgroundDrawable (BA ba, Drawable background)
        {
        mPopupWindow.setBackgroundDrawable (background);
        }

/**
    * Allows the popup window to extend beyond the bounds of the screen. By default the window is clipped to the screen boundaries. Setting this to false will allow windows to be accurately positioned.
    * If the popup is showing, calling this method will take effect only the next time the popup is shown or through a manual call to one of the update() methods.
    */
    public void setClippingEnabled (BA ba, boolean enabled)
        {
        mPopupWindow.setClippingEnabled (enabled);
        }

/**
    * Changes the popup's content. The content is represented by an instance of View. This method has no effect if called when the popup is showing.
    */
    public void setContentView (BA ba, android.view.View contentView)
        {
        mPopupWindow.setContentView (contentView);
        }

/**
    * (API 21+) Specifies the elevation in pixels for this popup window.
    */
    public void setElevation (BA ba, float elevation)
        {
        if (Build.VERSION.SDK_INT >= 21)
            {
            mPopupWindow.setElevation (elevation);
            }
        }

/**
    * (API 23+) Sets the enter transition to be used when the popup window is shown.
    */
    public void setEnterTransition (BA ba, Transition enterTransition)
        {
        if (Build.VERSION.SDK_INT >= 23)
            {
            mPopupWindow.setEnterTransition (enterTransition);
            }
        }

/**
    * (API 23+) Sets the exit transition to be used when the popup window is dismissed.
    */
    public void setExitTransition (BA ba, Transition exitTransition)
        {
        if (Build.VERSION.SDK_INT >= 23)
            {
            mPopupWindow.setEnterTransition (exitTransition);
            }
        }

/**
    * Changes the focusability of the popup window. When focusable, the window will grab the focus from the current focused widget if the popup contains a focusable View. By default a popup window is not focusable.
    * If the popup is showing, calling this method will take effect only the next time the popup is shown or through a manual call to one of the update() methods.
    */
    public void setFocusable (BA ba, boolean focusable)
        {
        mPopupWindow.setFocusable (focusable);
        }

/**
    * Sets the popup's requested height. May be a layout constant such as ViewGroup.LayoutParams.WRAP_CONTENT or ViewGroup.LayoutParams.MATCH_PARENT. The actual size of the popup may depend on other factors such as clipping and window layout.
    * If the popup is showing, calling this method will take effect the next time the popup is shown.
    */
    public void setHeight (BA ba, int height)
        {
        mPopupWindow.setHeight (height);
        }

/**
    * Set the flag on popup to ignore cheek press events; by default this flag is set to false which means the popup will not ignore cheek press dispatch events.
    * If the popup is showing, calling this method will take effect only the next time the popup is shown or through a manual call to one of the update() methods.
    */
    public void setIgnoreCheekPress (BA ba)
        {
        mPopupWindow.setIgnoreCheekPress ();
        }

/**
    * Control how the popup operates with an input method: one of INPUT_METHOD_FROM_FOCUSABLE, INPUT_METHOD_NEEDED, or INPUT_METHOD_NOT_NEEDED.
    * If the popup is showing, calling this method will take effect only the next time the popup is shown or through a manual call to one of the update() methods.
    */
    public void setInputMethodMode (BA ba, int mode)
        {
        mPopupWindow.setInputMethodMode (mode);
        }

/**
    * Controls whether the pop-up will be informed of touch events outside of its window. This only makes sense for pop-ups that are touchable but not focusable, which means touches outside of the window will be delivered to the window behind. The default is false.
    * If the popup is showing, calling this method will take effect only the next time the popup is shown or through a manual call to one of the update() methods.
    */
    public void setOutsideTouchable (BA ba, boolean touchable)
        {
        mPopupWindow.setOutsideTouchable (touchable);
        }

/**
    * (API 23+) Sets whether the popup window should overlap its anchor view when displayed as a drop-down.
    * If the popup is showing, calling this method will take effect only the next time the popup is shown.
    */
    public void setOverlapAnchor (BA ba, boolean overlapAnchor)
        {
        if (Build.VERSION.SDK_INT >= 23)
            {
            mPopupWindow.setOverlapAnchor (overlapAnchor);
            }
        }

/**
    * Sets the operating mode for the soft input area.
    */
    public void setSoftInputMode (BA ba, int mode)
        {
        mPopupWindow.setSoftInputMode (mode);
        }

/**
    * Allows the popup window to split touches across other windows that also support split touch. When this flag is false, the first pointer that goes down determines the window to which all subsequent touches go until all pointers go up. When this flag is true, each pointer (not necessarily the first) that goes down determines the window to which all subsequent touches of that pointer will go until that pointer goes up thereby enabling touches with multiple pointers to be split across multiple windows.
    */
    public void setSplitTouchEnabled (BA ba, boolean enabled)
        {
        mPopupWindow.setSplitTouchEnabled (enabled);
        }

/**
    * Changes the touchability of the popup window. When touchable, the window will receive touch events, otherwise touch events will go to the window below it. By default the window is touchable.
    * If the popup is showing, calling this method will take effect only the next time the popup is shown or through a manual call to one of the update() methods.
    */
    public void setTouchable (BA ba, boolean touchable)
        {
        mPopupWindow.setTouchable (touchable);
        }

/**
    * Sets the popup's requested width. May be a layout constant such as ViewGroup.LayoutParams.WRAP_CONTENT or ViewGroup.LayoutParams.MATCH_PARENT.
    * The actual size of the popup may depend on other factors such as clipping and window layout.
    * If the popup is showing, calling this method will take effect the next time the popup is shown.
    */
    public void setWidth (BA ba, int width)
        {
        mPopupWindow.setWidth (width);
        }

/**
    * (API 23+) Sets the layout type for this window. See WindowManager.LayoutParams.type for possible values.
    */
    public void setWindowLayoutType (BA ba, int layoutType)
        {
        if (Build.VERSION.SDK_INT >= 23)
            {
            mPopupWindow.setWindowLayoutType (layoutType);
            }
        }

/**
    * Displays the content view in a popup window anchored to the corner of another view. The window is positioned according to the specified gravity and offset by the specified x and y coordinates.
    * If there is not enough room on screen to show the popup in its entirety, this method tries to find a parent scroll view to scroll. If no parent view can be scrolled, the specified vertical gravity will be ignored and the popup will anchor itself such that it is visible.
    * If the view later scrolls to move anchor to a different location, the popup will be moved correspondingly.
    * xoff: A horizontal offset from the anchor in pixels. yoff: A vertical offset from the anchor in pixels.
    * gravity (API 19+) : Alignment of the popup relative to the anchor.
    * gravity (API 18-) : Not used.
    */
    public void showAsDropDown (BA ba, android.view.View anchor, int xoff, int yoff, int gravity)
        {
        if (Build.VERSION.SDK_INT >= 19)
            {
            mPopupWindow.showAsDropDown (anchor, xoff, yoff, gravity);
            }
        else
            {
            mPopupWindow.showAsDropDown (anchor, xoff, yoff);
            }
        if (mPanel.getWidth () < 0)
           {
           mPanel.setWidth (mPopupWindow.getWidth ());
           }
        if (mPanel.getHeight () < 0)
           {
           mPanel.setHeight (mPopupWindow.getHeight ());
           }
        }

/**
    * Display the content view in a popup window at the specified location. If the popup window cannot fit on screen, it will be clipped. See WindowManager.LayoutParams for more information on how gravity and the x and y parameters are related. Specifying a gravity of Gravity.NO_GRAVITY is similar to specifying Gravity.LEFT | Gravity.TOP.
    */
    public void showAtLocation (BA ba, android.view.View parent, int gravity, int x, int y)
        {
        mPopupWindow.showAtLocation (parent, gravity, x, y);
        }

/**
    * Updates the position and the dimension of the popup window. Calling this function also updates the window with the current popup state.
    * Width and height can be set to -1 to update location only.
    * force: true to reposition the window even if the specified position already seems to correspond to the LayoutParams, false to only reposition if needed.
    */
    public void update (BA ba, int x, int y, int width, int height, boolean force)
        {
        mPopupWindow.update (x, y, width, height, force);
        }

/**
    * Auxiliary function (outside PopupWindow class).
    */
    public static Object simpleRippleEffect (BA ba, int defaultColor, int pressedColor)
        {
        if (Build.VERSION.SDK_INT >= 21)
            {
            ColorStateList pressedColorDrawable = new ColorStateList
                (
                new int [ ] [ ]
                    {
                    new int [ ] { android.R.attr.state_pressed },
                    new int [ ] { android.R.attr.state_focused },
                    new int [ ] { android.R.attr.state_activated },
                    new int [ ] { }
                    },
                new int [ ]
                    {
                    pressedColor,
                    pressedColor,
                    pressedColor,
                    defaultColor
                    }
                );
            ColorDrawable defaultColorDrawable = new ColorDrawable (defaultColor);
            RippleDrawable result = new RippleDrawable (pressedColorDrawable, defaultColorDrawable, null);
            return result;
            }
        else
            {
            ColorDrawable pressedColorDrawable = new ColorDrawable (pressedColor);
            ColorDrawable defaultColorDrawable = new ColorDrawable (defaultColor);
            StateListDrawable states = new StateListDrawable ();
            states.addState (new int [] { android.R.attr.stateNotNeeded }, defaultColorDrawable);
            states.addState (new int [] { android.R.attr.state_pressed }, pressedColorDrawable);
            return states;
            }
        }
    }
